from Products.ZPatterns.Specialists import Specialist, manage_addSpecialistForm
from Globals import HTMLFile
import random, string
from DateTime.DateTime import DateTime
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
import CSV

class PropertiesHolder(SimpleItem, PropertyManager):
    """ Holds property description for FlatDatabase """
    
    _properties = ()


class FDSpecialist(Specialist):
    """ Store items of a flat database. """
    
    meta_type = "Flat Database"
    
    __ac_permissions__=(
        ('Manage properties', ('manage_addEntry',
                               'manage_delEntries',
                               'csv_importer',
                               'manage_DBPropertiesForm',
                               'manage_ImportCSVForm',
                               'manage_ImportCSV',
                               )),
        ('View', ('entryKeys',
                  'entryValues',
                  'entryItems',
                  'index_html',
                  'nameToPropName',
                  'sortBy',
                  'export_csv',               
                  )),
       )
        
    def __init__(self, id, title = ''):
        Specialist.__init__(self, id, title)
        self.database_properties = PropertiesHolder()
        
    manage_DBPropertiesForm = HTMLFile("www/DBPropertiesForm", globals())
    
    index_html = HTMLFile("www/index_html", globals())
        
    manage_ImportCSVForm = HTMLFile("www/ImportCSVForm", globals())
    
    manage_ImportCSV = HTMLFile("www/ImportCSV", globals())
        
    def entryKeys(self):
        """ Return keys for all entries in database """
        keys = []
        for rack in self.rackList:
            keys = keys + list(rack.getPersistentItemIDs())
        return keys
        
    def entryValues(self):
        """ Return all entries in database """
        return map(self.getItem, self.entryKeys())
        
    def entryItems(self):
        """ Return key, value pairs of entries in database """
        keys = self.entryKeys()
        return map(None, keys, map(self.getItem, keys))
    
    def manage_addEntry(self, REQUEST=None, RESPONSE=None):
        """ Add a new database entry """
        id = str(float(DateTime()))  + str(random.randint(0, 999999)) 
        entry = self.newItem(id)
        #for d in self.database_properties.propertyMap():
        #    entry.manage_addProperty(id=d['id'], type=d['type'], value='')
            
        if RESPONSE and REQUEST:
            RESPONSE.redirect(REQUEST.URL1 + '/' + id + '/manage')
        else:
            return entry
            
    def manage_delEntries(self, del_ids=[], REQUEST=None, RESPONSE=None):
        """ Delete entries """
        for id in del_ids:
            self.getItem(id).manage_delete()
        RESPONSE.redirect(REQUEST.URL1 + '/index_html')
               
    def sortBy(self, items, attr):
        """ Sort a list of dictionaries by a specific attribute """
        cmpfunc = lambda x, y, attr=attr: cmp(getattr(x, attr, ''), getattr(y, attr, ''))
        items.sort(cmpfunc)
        return items     
    
    def csv_importer(self,fileobj,REQUEST):
        """ Import CSV into our database """
        
        rec_count = 0    # how many lines have we imported
        
        if REQUEST.has_key('sep') and REQUEST['sep'] != '':
            sep = REQUEST['sep']
        else:    
            sep = '\t'
        
        csv = CSV.CSV()
        
        csv.input(fileobj.read(-1),1,0,sep,["#"])
        
        cols = csv.fields__title
        
        # Set columns as attributes of the database
        for col in cols:
            label = self.nameToPropName(col)
            if not self.database_properties.hasProperty(label):
                self.database_properties.manage_addProperty(label,col,'string',None)
        
        # Add all stored entries
        for entry in csv:
            
            # create the entry
            Item = self.manage_addEntry(self)
            
            # Set the entries properties
            for i in range(len(entry)):
                setattr(Item, self.nameToPropName(cols[i]),entry[i])    
                
            rec_count = rec_count + 1
            
        return rec_count
    
    def csv_exporter(self):
        """ Export database to CSV """
        result = ""
        csv = CSV.CSV()
        d = self.database_properties
        
        propertyNames = d.propertyIds()
        for p in propertyNames:
            if p[0:4] != 'col_': propertyNames.remove(p)
              
        csv.fields__title = map(lambda k, d=d: d.getProperty(k), propertyNames)
        csv.fields__title__have = 1

        for entry in self.entryValues():
            properties = []
            for id in propertyNames:
                properties.append(entry.getProperty(id))
            csv_entry = CSV.Entry(properties)
            csv.append(csv_entry)
            
        return csv.output()
    
    def export_csv(self, REQUEST, RESPONSE):
        """ csv_exporter wrapper that deals with mime-types, etc. """
        RESPONSE.setHeader('Content-Type', 'application/csv')
        RESPONSE.setHeader('Content-Disposition', 
                           'inline; filename="%s.csv"' % self.id)
        return self.csv_exporter()
        
    def nameToPropName(self, property):
        """ Convert 'property name' to 'col_property_name' """
        name = string.join(['col',property],'_')
        name = string.replace(name, ' ', '_')
        return name


def manage_addSpecialist(self, id, title='', rack='', REQUEST=None):
    """Add a Specialist"""
    ob = FDSpecialist(id, title)
    self._setObject(id, ob)
    ob = ob.__of__(self)
    
    if rack:
        ob.RacksGroup._constructPlugIn(rack, id='defaultRack')
        ob.defaultRack.manage_setStorage(
                zclass="Products.FlatDatabase.Entry/EntryDataSkin"
                )
    
    if REQUEST: return self.manage_main(self,REQUEST,update_menu=1)



def initialize(context):

    context.registerClass(
        FDSpecialist,
        permission = 'Add Flat Databases',
        constructors = (manage_addSpecialistForm,
                        manage_addSpecialist),
        icon = 'www/fdspecialist.gif',
    )
